# Event sources

An event source is a Kubernetes custom resource (CR), created by a developer or cluster administrator, that acts as a link between an event producer and an event _sink_.
A sink can be a k8s service, including Knative Services, a Channel, or a Broker that receives events from an event source.

Event sources are created by instantiating a CR from a Source object.
The Source object defines the arguments and parameters needed to instantiate a CR.

All Sources are part of the `sources` category.

=== "kn"
    You can list existing event sources on your cluster by entering the kn command:

    ```bash
    kn source list
    ```

=== "kubectl"
    You can list existing event sources on your cluster by entering the command:

    ```bash
    kubectl get sources
    ```

!!! note
    Event Sources that import events from other messaging technologies such as Kafka or RabbitMQ are not responsible for setting [Optional Attributes](https://github.com/cloudevents/spec/blob/v1.0.2/cloudevents/spec.md#optional-attributes) such as the `datacontenttype`. This is a responsibility of the original event producer; the Source just appends attributes if they exist.

## Knative Sources

| Name | Status | Maintainer | Description |
| -- | -- | -- | -- |
| [APIServerSource](apiserversource/README.md) | Stable | Knative  | Brings Kubernetes API server events into Knative. The APIServerSource fires a new event each time a Kubernetes resource is created, updated or deleted. |
| [Apache CouchDB](https://github.com/knative-extensions/eventing-couchdb/blob/main/source)                                                   | Alpha | Knative    | Brings [Apache CouchDB](https://couchdb.apache.org/) messages into Knative.  |
| [Apache Kafka](kafka-source/README.md)       | Stable  | Knative    | Brings [Apache Kafka](https://kafka.apache.org/) messages into Knative. The KafkaSource reads events from an Apache Kafka Cluster, and passes these events to a sink so that they can be consumed. See the [Kafka Source](https://github.com/knative-extensions/eventing-kafka/blob/main/pkg/source) example for more details.  |
| [CephSource](https://github.com/knative-extensions/eventing-ceph) | Beta | Knative    | The Ceph source converts bucket notifications from [Ceph format](https://docs.ceph.com/docs/master/radosgw/notifications/#events) into CloudEvents format, and inject them into Knative. Conversion logic follow the one described for [AWS S3](https://github.com/cloudevents/spec/blob/master/adapters/aws-s3.md) bucket notifications. |
| [ContainerSource](../custom-event-source/containersource/README.md)                    | Stable | Knative    | The ContainerSource instantiates container image(s) that can generate events until the ContainerSource is deleted. This may be used, for example, to poll an FTP server for new files or generate events at a set time interval. Given a `spec.template` with at least a container image specified, the ContainerSource keeps a Pod running with the specified image(s). `K_SINK` (destination address) and `KE_CE_OVERRIDES` (JSON CloudEvents attributes) environment variables are injected into the running image(s). It is used by multiple other Sources as underlying infrastructure. Refer to the [Container Source](../custom-event-source/containersource/README.md) example for more details.  |
| [GitHub](https://github.com/knative/docs/tree/main/code-samples/eventing/github-source)            | Beta | Knative    | Registers for events of the specified types on the specified GitHub organization or repository, and brings those events into Knative. The GitHubSource fires a new event for selected [GitHub event types](https://developer.github.com/v3/activity/events/types/). See the [GitHub Source](https://github.com/knative/docs/tree/main/code-samples/eventing/github-source) example for more details.  |
| [GitLab](https://github.com/knative/docs/tree/main/code-samples/eventing/gitlab-source) | Beta | Knative    | Registers for events of the specified types on the specified GitLab repository, and brings those events into Knative. The GitLabSource creates a webhooks for specified [event types](https://docs.gitlab.com/ee/user/project/integrations/webhooks.html#events), listens for incoming events, and passes them to a consumer. See the [GitLab Source](https://github.com/knative/docs/tree/main/code-samples/eventing/gitlab-source) example for more details.  |
| [KogitoSource](https://github.com/knative-extensions/eventing-kogito) |  Alpha | Knative    | An implementation of the [Kogito Runtime](https://docs.jboss.org/kogito/release/latest/html_single/#proc-kogito-deploying-on-kubernetes_kogito-deploying-on-openshift) custom resource managed by the [Kogito Operator](https://github.com/kiegroup/kogito-operator). |
| [PingSource](ping-source/README.md) | Stable | Knative    | Produces events with a fixed payload on a specified [Cron](https://en.wikipedia.org/wiki/Cron) schedule. See the [Ping Source](ping-source/README.md) example for more details. |
| [RabbitMQ](https://github.com/knative-extensions/eventing-rabbitmq) | Stable | Knative | Brings [RabbitMQ](https://www.rabbitmq.com/) messages into Knative.
[RedisSource](https://github.com/knative-extensions/eventing-redis/tree/{{version}}/source) | Alpha | Knative | Brings Redis Stream into Knative.
| [SinkBinding](../custom-event-source/sinkbinding/README.md) | Stable | Knative    | The SinkBinding can be used to author new event sources using any of the familiar compute abstractions that Kubernetes makes available (e.g. Deployment, Job, DaemonSet, StatefulSet), or Knative abstractions (e.g. Service, Configuration). SinkBinding provides a framework for injecting `K_SINK` (destination address) and `K_CE_OVERRIDES` (JSON cloudevents attributes) environment variables into any Kubernetes resource which has a `spec.template` that looks like a Pod (aka PodSpecable). See the [SinkBinding](../custom-event-source/sinkbinding/README.md) example for more details. |

## Third-Party Sources

| Name | Status | Maintainer | Description
|--|--|--|--|
[Amazon CloudWatch](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSCloudWatchSource) | Stable | TriggerMesh | Collects metrics from [Amazon CloudWatch](https://aws.amazon.com/cloudwatch/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awscloudwatch/))
[Amazon CloudWatch Logs](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSCloudWatchLogsSource) | Stable | TriggerMesh | Subscribes to log events from an [Amazon CloudWatch Logs](https://aws.amazon.com/cloudwatch/) stream. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awscloudwatchlogs/))
[AWS CodeCommit](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSCodeCommitSource) | Stable | TriggerMesh | Registers for events emitted by an [AWS CodeCommit](https://aws.amazon.com/codecommit/) source code repository. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awscodecommit/))
[Amazon Cognito Identity](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSCognitoIdentitySource) | Stable | TriggerMesh | Registers for events from [Amazon Cognito](https://aws.amazon.com/cognito/) identity pools. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awscognitoidentity/))
[Amazon Cognito User](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSCognitoUserPoolSource) | Stable | TriggerMesh | Registers for events from [Amazon Cognito](https://aws.amazon.com/cognito/) user pools. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awscognitouserpool/))
[Amazon DynamoDB](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSDynamoDBSource) | Stable | TriggerMesh | Reads records from an [Amazon DynamoDB](https://aws.amazon.com/dynamodb/) stream. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awsdynamodb/))
[Amazon Kinesis](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSKinesisSource) | Stable | TriggerMesh | Reads records from an [Amazon Kinesis](https://aws.amazon.com/kinesis/) stream. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awskinesis/))
[Amazon S3](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSS3Source) | Stable | TriggerMesh | Subscribes to event notifications from an [Amazon S3](https://aws.amazon.com/s3/) bucket. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awss3/))
[Amazon SNS](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSSNSSource) | Stable | TriggerMesh | Subscribes to messages from an [Amazon SNS](https://aws.amazon.com/sns/) topic. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awssns/))
[Amazon SQS](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AWSSQSSource) | Stable | TriggerMesh | Consumes messages from an [Amazon SQS](https://aws.amazon.com/sqs/) queue. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/awssqs/))
[Apache Camel](https://camel.apache.org/camel-kamelets/latest) | Stable | Apache Software Foundation    | Enables use of [Apache Camel](https://github.com/apache/camel) components for pushing events into Knative. Camel sources are now provided via [Kamelets](https://camel.apache.org/camel-kamelets/latest/) as part of the [Apache Camel K](https://camel.apache.org/camel-k/latest/installation/installation.html) project. |
[Azure Activity Logs](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureActivityLogsSource) | Stable | TriggerMesh | Capture activity logs from [Azure Activity Logs](https://docs.microsoft.com/en-us/azure/azure-monitor/essentials/activity-log). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureactivitylogs/))
[Azure Blob Storage](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureBlobStorageSource) | Stable | TriggerMesh | Subscribes to events from an [Azure Blob Storage](https://azure.microsoft.com/en-us/services/storage/blobs/) account. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureblobstorage/))
[Azure Event Grid](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureEventGridSource) | Stable | TriggerMesh | Retrieves events from [Azure Event Grid](https://azure.microsoft.com/en-us/services/event-grid/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureeventgrid/))
[Azure Event Hubs](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureEventHubSource) | Stable | TriggerMesh | Consumes events from [Azure Event Hubs](https://azure.microsoft.com/en-us/services/event-hubs/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureeventhubs/))
[Azure IoT Hub](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureIOTHubSource) | Stable | TriggerMesh | Consumes event from [Azure IoT Hub](https://azure.microsoft.com/en-us/services/iot-hub/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureiothub/))
[Azure Queue Storage](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureQueueStorageSource) | Stable | TriggerMesh | Retrieves messages from [Azure Queue Storage](https://azure.microsoft.com/en-us/services/storage/queues/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azurequeuestorage/))
[Azure Service Bus Queues](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureServiceBusQueueSource) | Stable | TriggerMesh | Consumes messages from an [Azure Service Bus](https://azure.microsoft.com/en-us/services/service-bus/) queue. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureservicebusqueue/))
[Azure Service Bus Topics](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.AzureServiceBusTopicSource) | Stable | TriggerMesh | Subscribes to messages from an [Azure Service Bus](https://azure.microsoft.com/en-us/services/service-bus/) topic. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/azureservicebustopic/))
[Debezium](https://debezium.io) | Alpha | Debezium | Consume database changes as CloudEvents in Knative. ([knative configuration](https://github.com/cab105/debezium-knative-integration-sample))
[Direktiv](https://github.com/direktiv/direktiv-knative-source/tree/main/cmd/direktiv-source) | Alpha | Direktiv | Receive events from [Direktiv](https://github.com/direktiv/direktiv).
[DockerHubSource](https://github.com/tom24d/eventing-dockerhub) | Alpha | None | Retrieves events from [Docker Hub Webhooks](https://docs.docker.com/docker-hub/webhooks/) and transforms them into CloudEvents for consumption in Knative.
[Google Cloud Audit Logs](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.GoogleCloudAuditLogsSource) | Stable | TriggerMesh | Captures audit logs from [Google Cloud Audit Logs](https://cloud.google.com/logging/docs/audit/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/googlecloudauditlogs/))
[Google Cloud Billing](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.GoogleCloudBillingSource) | Stable | TriggerMesh | Captures budget notifications from [Google Cloud Billing](https://cloud.google.com/billing/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/googlecloudbilling/))
[Google Cloud Pub/Sub](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.GoogleCloudPubSubSource) | Stable | TriggerMesh | Subscribes to messages from a [Google Cloud Pub/Sub](https://cloud.google.com/pubsub/) topic. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/googlecloudpubsub/))
[Google Cloud Source Repositories](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.GoogleCloudSourceRepositoriesSource) | Stable | TriggerMesh | Consumes events from [Google Cloud Source Repositories](https://cloud.google.com/source-repositories). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/googlecloudrepositories/))
[Google Cloud Storage](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.GoogleCloudStorageSource) | Stable | TriggerMesh | Captures change notifications from a [Google Cloud Storage](https://cloud.google.com/storage) bucket. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/googlecloudstorage/))
[HTTP Poller](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.HTTPPollerSource) | Stable | TriggerMesh | Periodically pulls events from an HTTP/S URL. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/httppoller/))
[Oracle Cloud Infrastructure](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.OCIMetricsSource) | Stable | TriggerMesh | Retrieves metrics from [Oracle Cloud Infrastructure](https://www.oracle.com/cloud/). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/ocimetrics/))
[Salesforce](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.SalesforceSource) | Stable | TriggerMesh | Consumes events from a [Salesforce](https://www.salesforce.com) channel. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/salesforce/))
[Slack](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.SlackSource) | Stable | TriggerMesh | Subscribes to events from [Slack](https://slack.com). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/slack/))
[Twilio](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.TwilioSource) | Supported | TriggerMesh | Receive events from [Twilio](https://twilio.com). ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/twilio/))
[VMware](https://github.com/vmware-tanzu/sources-for-knative/blob/main/README.md) | Alpha | VMware | Brings [vSphere](https://www.vmware.com/products/vsphere.html) events into Knative.
[Webhook](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.WebhookSource) | Stable | TriggerMesh | Ingest events from a webhook using HTTP. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/webhook/))
[Zendesk](https://docs.triggermesh.io/latest/reference/sources/#sources.triggermesh.io/v1alpha1.ZendeskSource) | Stable | TriggerMesh | Subscribes to events from Zendesk. ([installation](https://docs.triggermesh.io/latest/installation/)) ([example](https://docs.triggermesh.io/latest/sources/zendesk/))

## Additional resources

- If your code needs to send events as part of its business logic and doesn't fit the model of a Source, consider [feeding events directly to a Broker](https://knative.dev/docs/eventing/broker/).
- For more information about using `kn` Source related commands, see the [`kn source` reference documentation](https://github.com/knative/client/blob/main/docs/cmd/kn_source.md).
